@@ -15,10 +15,11 @@ use crate::{
         Confidence, Finding, Fix, FixDisposition, Severity,
         location::{Locatable as _, Routable},
     },
-    github_api::{self, ComparisonStatus},
+    github::{self, ComparisonStatus},
     models::{
         StepCommon,
         uses::RepositoryUsesExt as _,
+        version::Version,
         workflow::{ReusableWorkflowCallJob, Workflow},
     },
     registry::input::InputKey,
@@ -30,7 +31,7 @@ use yamlpatch::{Op, Patch};
 pub const IMPOSTOR_ANNOTATION: &str = "uses a commit that doesn't belong to the specified org/repo";
 
 pub(crate) struct ImpostorCommit {
-    pub(crate) client: github_api::Client,
+    pub(crate) client: github::Client,
 }
 
 audit_meta!(
@@ -114,36 +115,22 @@ impl ImpostorCommit {
 
         // If we've made it here, the commit isn't present in any commit or tag's history,
         // strongly suggesting that it's an impostor.
-        tracing::warn!(
-            "strong impostor candidate: {head_ref} for {org}/{repo}",
-            org = uses.owner,
-            repo = uses.repo
-        );
         Ok(true)
     }
 
-    /// Get the latest tag for a repository, preferring semantic version tags
-    fn get_latest_tag(&self, uses: &RepositoryUses) -> Result<Option<String>> {
+    /// Return the highest semantically versioned tag in the repository.
+    fn get_highest_tag(&self, uses: &RepositoryUses) -> Result<Option<String>> {
         let tags = self.client.list_tags(&uses.owner, &uses.repo)?;
 
-        // Prefer semantic version tags (starting with 'v') over other tags
-        let semver_tags: Vec<_> = tags
+        // Filter tags down to those that can be parsed as semantic versions,
+        // get the highest one, and return its original string representation.
+        let highest_tag = tags
             .iter()
-            .filter(|tag| {
-                tag.name.starts_with('v')
-                    && tag.name.chars().nth(1).is_some_and(|c| c.is_ascii_digit())
-            })
-            .collect();
+            .filter_map(|tag| Version::parse(&tag.name).ok())
+            .max()
+            .map(|vers| vers.raw().to_string());
 
-        let latest_tag = if !semver_tags.is_empty() {
-            // Use the first semver tag (GitHub returns tags in chronological order, newest first)
-            semver_tags.first().map(|tag| &tag.name)
-        } else {
-            // Fallback to the first tag if no semver tags are found
-            tags.first().map(|tag| &tag.name)
-        };
-
-        Ok(latest_tag.cloned())
+        Ok(highest_tag)
     }
 
     /// Create a fix for an impostor commit by replacing it with the latest tag
@@ -175,7 +162,7 @@ impl ImpostorCommit {
         route: yamlpath::Route<'doc>,
     ) -> Option<Fix<'doc>> {
         // Get the latest tag for this repository
-        let latest_tag = match self.get_latest_tag(uses) {
+        let latest_tag = match self.get_highest_tag(uses) {
             Ok(Some(tag)) => tag,
             Ok(None) => {
                 tracing::warn!(
@@ -348,10 +335,9 @@ jobs:
         let state = crate::state::AuditState {
             no_online_audits: false,
             gh_client: Some(
-                crate::github_api::Client::new(
-                    &crate::github_api::GitHubHost::Standard("github.com".to_string()),
-                    &crate::github_api::GitHubToken::new(&std::env::var("GH_TOKEN").unwrap())
-                        .unwrap(),
+                crate::github::Client::new(
+                    &crate::github::GitHubHost::Standard("github.com".to_string()),
+                    &crate::github::GitHubToken::new(&std::env::var("GH_TOKEN").unwrap()).unwrap(),
                     "/tmp".into(),
                 )
                 .unwrap(),
@@ -408,10 +394,9 @@ jobs:
         let state = crate::state::AuditState {
             no_online_audits: false,
             gh_client: Some(
-                crate::github_api::Client::new(
-                    &crate::github_api::GitHubHost::Standard("github.com".to_string()),
-                    &crate::github_api::GitHubToken::new(&std::env::var("GH_TOKEN").unwrap())
-                        .unwrap(),
+                crate::github::Client::new(
+                    &crate::github::GitHubHost::Standard("github.com".to_string()),
+                    &crate::github::GitHubToken::new(&std::env::var("GH_TOKEN").unwrap()).unwrap(),
                     "/tmp".into(),
                 )
                 .unwrap(),
